"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.get_curve_growth = exports.get_dfuse_curve = void 0;
const dfuse_1 = require("./dfuse");
function get_dfuse_curve(client, symcode, block_num) {
    return __awaiter(this, void 0, void 0, function* () {
        return dfuse_1.stateTableRow(client, "curve.sx", "curve.sx", "pairs", symcode, block_num);
    });
}
exports.get_dfuse_curve = get_dfuse_curve;
function get_curve_growth(client, symcode, block_num, block_num_delta = 172800, trade_fee = 4, protocol_fee = 0) {
    return __awaiter(this, void 0, void 0, function* () {
        const block_num_previous = block_num - block_num_delta;
        const curve = yield get_dfuse_curve(client, symcode, block_num);
        const curve_previous = yield get_dfuse_curve(client, symcode, block_num_previous);
        // trade volume stats
        const volume0 = toNumber(curve.volume0) - toNumber(curve_previous.volume0);
        const volume1 = toNumber(curve.volume1) - toNumber(curve_previous.volume1);
        const volume = volume0 + volume1;
        const trades = curve.trades - curve_previous.trades;
        // utilization calculated by how much traded vs. reserve
        const reserve0 = toNumber(curve.reserve0.quantity);
        const reserve1 = toNumber(curve.reserve1.quantity);
        const tvl = reserve0 + reserve1;
        const utilization = volume / tvl;
        // reserve growth
        const reserve0_previous = toNumber(curve_previous.reserve0.quantity);
        const reserve1_previous = toNumber(curve_previous.reserve1.quantity);
        const tvl_previous = reserve0_previous + reserve1_previous;
        const tvl_growth = tvl - tvl_previous;
        // calculate real growth APY
        const trade_fees = volume * trade_fee / 10000;
        const protocol_fees = volume * protocol_fee / 10000;
        const fees = trade_fees + protocol_fees;
        const average_tvl = (tvl_previous + tvl) / 2;
        const apy_average_revenue = fees * 365 / average_tvl;
        const apy_realtime_revenue = fees * 365 / tvl;
        // value per share APY
        const virtual_price = curve.virtual_price;
        const virtual_price_growth = virtual_price - curve_previous.virtual_price; // daily growth
        const growth = tvl * virtual_price_growth; // approximate fees based on growth
        return {
            // block information
            block_num,
            block_num_previous,
            block_num_delta,
            // contract values
            id: curve.id,
            reserve0: curve.reserve0,
            reserve1: curve.reserve1,
            liquidity: curve.liquidity,
            amplifier: curve.amplifier,
            virtual_price: curve.virtual_price,
            price0_last: curve.price0_last,
            price1_last: curve.price1_last,
            volume0: curve.volume0,
            volume1: curve.volume1,
            trades: curve.trades,
            last_updated: curve.last_updated,
            // 24h computed values
            apy_average_revenue,
            apy_realtime_revenue,
            volume,
            tvl,
            tvl_growth,
            utilization,
            fees,
            growth,
            virtual_price_growth,
        };
    });
}
exports.get_curve_growth = get_curve_growth;
function toNumber(quantity) {
    return Number(quantity.split(" ")[0] || 0);
}
