"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.get_inverse_price = exports.get_price = void 0;
const eos_common_1 = require("eos-common");
const bancor_1 = require("./bancor");
const get_uppers_1 = require("./get_uppers");
const checks_1 = require("./checks");
function get_price(quantity, symcode, tokens, settings) {
    // params
    const _quantity = new eos_common_1.Asset(quantity);
    const in_amount = eos_common_1.asset_to_number(_quantity);
    const base = _quantity.symbol.code();
    const quote = new eos_common_1.SymbolCode(symcode);
    // validation
    checks_1.check_quantity(quantity);
    // upper limits
    const [base_upper, quote_upper] = get_uppers_1.get_uppers(base, quote, tokens, settings);
    // Bancor V1 Formula
    return bancor_1.get_bancor_output(base_upper, quote_upper, in_amount);
}
exports.get_price = get_price;
function get_inverse_price(out, symcode, tokens, settings) {
    // params
    const _out = new eos_common_1.Asset(out);
    const base = _out.symbol.code();
    const quote = eos_common_1.symbol_code(symcode);
    const quote_sym = tokens[quote.to_string()].balance.symbol;
    // validation
    checks_1.check_quantity(_out);
    // uppers & pegged
    const [base_upper, quote_upper] = get_uppers_1.get_uppers(base, quote, tokens, settings);
    // Bancor V1 Formula
    const in_amount = bancor_1.get_bancor_input(base_upper, quote_upper, eos_common_1.asset_to_number(_out));
    return eos_common_1.number_to_asset(in_amount, quote_sym);
}
exports.get_inverse_price = get_inverse_price;
