"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.get_usdx_growth = exports.get_dfuse_usdx = void 0;
const dfuse_1 = require("./dfuse");
function get_dfuse_usdx(client, block_num) {
    return __awaiter(this, void 0, void 0, function* () {
        return dfuse_1.stateTable(client, "usdx.sx", "usdx.sx", "liquidity", block_num);
    });
}
exports.get_dfuse_usdx = get_dfuse_usdx;
function get_usdx_growth(client, block_num, block_num_delta = 172800) {
    return __awaiter(this, void 0, void 0, function* () {
        const block_num_previous = block_num - block_num_delta;
        const usdx = yield get_dfuse_usdx(client, block_num);
        const usdx_previous = yield get_dfuse_usdx(client, block_num_previous);
        // total value locked growth
        const tvl = usdx.deposit / 10000;
        const tvl_previous = usdx_previous.deposit / 10000;
        const tvl_growth = tvl - tvl_previous;
        // value per share APY
        const virtual_price = usdx.virtual_price || usdx.deposit / toNumber(usdx.supply.quantity);
        const virtual_price_previous = usdx_previous.virtual_price || usdx_previous.deposit / toNumber(usdx_previous.supply.quantity) / 10000;
        const virtual_price_growth = virtual_price - virtual_price_previous; // daily growth
        // calculate real growth APY
        const growth = tvl * virtual_price_growth; // approximate fees based on growth
        const average_tvl = (tvl_previous + tvl) / 2;
        const apy_average_revenue = growth * 365 / average_tvl;
        const apy_realtime_revenue = growth * 365 / tvl;
        return {
            // block information
            block_num,
            block_num_previous,
            block_num_delta,
            // contract values
            contract: usdx.contract,
            mid: usdx.mid,
            reserve0: usdx.reserve0,
            reserve1: usdx.reserve1,
            supply: usdx.supply,
            price0: usdx.price0,
            price1: usdx.price1,
            deposit: usdx.deposit,
            last_updated: usdx.last_updated,
            virtual_price: usdx.virtual_price,
            // 24h computed values
            apy_average_revenue,
            apy_realtime_revenue,
            tvl,
            tvl_growth,
            growth,
            virtual_price_growth,
        };
    });
}
exports.get_usdx_growth = get_usdx_growth;
function toNumber(quantity) {
    return Number(quantity.split(" ")[0] || 0);
}
